libname library '.';
proc format library=library;
   value sex 1='Male' 2='Female';
   value age 1='Fawn' 2='Yearling' 3='Adult' 4='Unknown Adult' ;
   value fate 0='Lived 15 June' 1='Coyote' 2='Bobcat' 3='Lion'
         4='Eagle' 5='Unknown Predator' 6='Starvation'
         7='Illegal kill' 8='Road kill' 9='Hunter kill'
         10='Accident' 11='Undetermined mortality' 12='No knowledge'
         13='Transmitter failed' 14='Dead battery'
         15='Collar dropped off' 16='Terminated data collection' ;
title 'Piceance Deer Study -- Fawn data from C-b for 1980-81 Winter';
data fawn;
 informat capture returned yymmdd6.;
 input id $ 1-8 sex 10 age 12 @14 capture
   @ 39 returned fate 48-49;
 format sex sex.;
 format age age.;
 format fate fate.;
 format capture returned date7.;
if (fate = 0) then returned=mdy(6,15,year(capture)+1);
    label  id='Animal identification'
           sex='Animal sex'
           age='Animal age at capture'
           capture='Date of capture'
           returned='Date of recovery'
           fate='Code for presumed cause of death';
cards;
191550AT 2 1 801211                   801229 2 11
191590AT 2 1 801213                   810116 1 01
191360AT 2 1 801213                   810123 2 01
191560AT 2 1 810116                   810128 3 01
191390AT 2 1 810115                   810130 4 05
191400AT 2 1 801215                   810202 3 11
191640AT 2 1 810115                   810211 3 01
191650AT 2 1 810123                   810211 3 11
191540AT 2 1 810114                   810218 1 01
191460AT 2 1 810116                   810222 2 01
191380AT 2 1 810119                   810304 1 01
191530AT 2 1 801126                   810308 3 01
191730AT 2 1 810214                   810312 1 01
191410AT 2 1 810214                   810316 3 11
191600AT 2 1 810117                   810318 4 11
191620AT 2 1 810118                   810322 1 01
191710AT 2 1 810120                   810329 5 07
191630AT 2 1 810120                   810402 3 01
191350AT 2 1 801216                   810403 5 11
191440AT 2 1 801123                   810406 3 01
191370AT 2 1 801217                     .    . 00
191430AT 2 1 801209                     .    . 00
191480AT 2 1 810215                     .    . 00
191520AT 2 1 801214                     .    . 00
191570AT 2 1 810213                     .    . 00
191720AT 2 1 810214                     .    . 00
;
proc print;
data temp;
   set fawn;
   drop id sex age capture returned fate;
   type='Capture';
   day=capture;
   format day date7.;
   output;
   if fate < 1 then return;
   else if fate > 11 then type='Censored';
   else type='Died';
   day=returned;
   output;
proc print;
proc sort; by day;
data kminput;
   retain norisk nodead nocensor noadded 0 firstday;
   set temp; by day;
   if _N_ = 1 then firstday=day;
   if type='Capture' then noadded=noadded+1;
   else if type='Died' then nodead=nodead+1;
   else if type='Censored' then nocensor=nocensor+1;
   if last.day then do;
      time=day-firstday;
      keep time norisk nodead nocensor noadded;
      if time > 0 then output;
      norisk=norisk+noadded-nodead-nocensor;
      nodead=0; nocensor=0; noadded=0;
      end;
proc print;
data kaplan;
   retain Shat 1 Greensum 0;
   set kminput;
   Shat=Shat*(1 - nodead/norisk);
   Greensum = Greensum + nodead/(norisk*(norisk-nodead));
   GvarShat = Shat*Shat*Greensum;
   GseShat = sqrt(GvarShat);
   varShat = Shat**2*(1 - Shat)/norisk;
   seShat = sqrt(varShat);
   lci = Shat - 1.96*GseShat;
   uci = Shat + 1.96*GseShat;
   output;
proc print;
proc plot;
   plot Shat*time='*' lci*time='-' uci*time='-' / overlay;
run;
