data coxoak72;
* Data from Table 7.2, Cox and Oakes (1984), Page 106.;
   title 'Table 7.2, Cox and Oakes (1984), Page 106';
   input time norisk0 nodead0 nocenor0 noadd0
              norisk1 nodead1 nocenor1 noadd1;
   cards;
  1 16 0 0 0 17 2 0 0
  2 16 1 0 0 15 0 0 0
  3 15 3 0 0 15 0 0 0
  4 12 3 0 0 15 1 0 0
  5  9 0 0 0 14 1 0 0
  7  9 1 0 0 13 0 0 0
  8  8 1 0 0 13 0 0 0
 16  7 1 0 0 13 1 0 0
 17  6 1 0 0 12 0 0 0
 22  5 1 0 0 12 1 0 0
 26  4 0 0 0 11 1 0 0
 30  4 1 0 0 10 0 0 0
 39  3 0 0 0 10 1 0 0
 43  3 1 0 0  9 0 0 0
 56  2 1 0 0  9 1 0 0
 65  1 1 0 0  8 2 0 0
100  0 0 0 0  6 1 0 0
108  0 0 0 0  5 1 0 0
121  0 0 0 0  4 1 0 0
134  0 0 0 0  3 1 0 0
143  0 0 0 0  2 1 0 0
156  0 0 0 0  1 1 0 0
;
data logrank;
   retain sumd1 0 sumEd0 0 sumEd1 0 svar1d1 0 svar2d1 0;
   retain Shat0 Shat1 1 Grensum0 Grensum1 0;
   set coxoak72 end=lastobs;
   * Compute the Kaplan-Meier Product Limit Estimator for
     each group, so that plots can be generated;
   if norisk0 > 0 then
      Shat0=Shat0*(1 - nodead0/norisk0);
   if norisk1 > 0 then
      Shat1=Shat1*(1 - nodead1/norisk1);
   if norisk0 > 0 & (norisk0 - nodead0) > 0 then
      Grensum0 = Grensum0 + nodead0/(norisk0*(norisk0-nodead0));
   if norisk1 > 0 & (norisk1 - nodead1) > 0 then
      Grensum1 = Grensum1 + nodead1/(norisk1*(norisk1-nodead1));
   seShat0 = sqrt(Shat0*Shat0*Grensum0);
   seShat1 = sqrt(Shat1*Shat1*Grensum1);
   lci0 = Shat0 - 1.96*seShat0;
   lci1 = Shat1 - 1.96*seShat1;
   uci0 = Shat0 + 1.96*seShat0;
   uci1 = Shat1 + 1.96*seShat1;
   keep time Shat0 Shat1 lci0 lci1 uci0 uci1;
   output;
   sumd1=sumd1+nodead1;
   if (norisk0 + norisk1) > 0 then do;
      sumEd0=sumEd0 + (nodead0 + nodead1)*norisk0/(norisk0 + norisk1);
      sumEd1=sumEd1 + (nodead0 + nodead1)*norisk1/(norisk0 + norisk1);
      if (norisk0 + norisk1) > 1 then
         svar1d1=svar1d1 + (nodead0 + nodead1)*norisk0*norisk1*
                  (norisk0 + norisk1 - nodead0 -nodead1)
                /((norisk0 + norisk1)**2*(norisk0 + norisk1 - 1));
         svar2d1=svar2d1 + (norisk0*norisk1*(nodead0 + nodead1))
                / (norisk0 + norisk1)**2;
   end;
   if lastobs then do;
      chisq1 = (sumd1 - sumEd1)**2/svar1d1;
      chisq2 = (sumd1 - sumEd1)**2/svar2d1;
      chisq3 = (sumd1 - sumEd1)**2 * (1/sumEd0 + 1/sumEd1);
      put sumd1= sumEd1= svar1d1= svar2d1=;
      put chisq1= chisq2= chisq3=;
      probchi1 = 1 - probchi(chisq1,1);
      probchi2 = 1 - probchi(chisq2,1);
      probchi3 = 1 - probchi(chisq3,1);
      put probchi1= probchi2= probchi3=;
      end;
proc print;
proc plot;
   plot Shat0*time='0' lci0*time='-' uci0*time='-'
   Shat1*time='1' lci1*time='+' uci1*time='+'
   / overlay;
data fall8586;
   * Bob-white quail data from Pollock;
   title 'Bob-white quail data from Pollock, 1985 vs. 1986';
   input time norisk0 nodead0 nocenor0 noadd0
              norisk1 nodead1 nocenor1 noadd1;
   cards;
1  7 1 0 0  7 0 1 0
2  6 0 0 2  6 0 0 5
3  8 0 0 5 11 1 0 0
4 13 0 1 6 10 0 0 6
5 18 0 0 0 16 1 0 0
6 18 0 0 0 15 0 0 0
7 18 0 0 0 15 1 0 0
8 18 0 0 0 14 0 0 0
9 18 0 0 1 14 3 0 0
;
data logrank;
   retain sumd1 0 sumEd0 0 sumEd1 0 svar1d1 0 svar2d1 0;
   retain Shat0 Shat1 1 Grensum0 Grensum1 0;
   set fall8586 end=lastobs;
   * Compute the Kaplan-Meier Product Limit Estimator for
     each group, so that plots can be generated;
   if norisk0 > 0 then
      Shat0=Shat0*(1 - nodead0/norisk0);
   if norisk1 > 0 then
      Shat1=Shat1*(1 - nodead1/norisk1);
   if norisk0 > 0 & (norisk0 - nodead0) > 0 then
      Grensum0 = Grensum0 + nodead0/(norisk0*(norisk0-nodead0));
   if norisk1 > 0 & (norisk1 - nodead1) > 0 then
      Grensum1 = Grensum1 + nodead1/(norisk1*(norisk1-nodead1));
   seShat0 = sqrt(Shat0*Shat0*Grensum0);
   seShat1 = sqrt(Shat1*Shat1*Grensum1);
   lci0 = Shat0 - 1.96*seShat0;
   lci1 = Shat1 - 1.96*seShat1;
   uci0 = Shat0 + 1.96*seShat0;
   uci1 = Shat1 + 1.96*seShat1;
   keep time Shat0 Shat1 lci0 lci1 uci0 uci1;
   output;
   sumd1=sumd1+nodead1;
   if (norisk0 + norisk1) > 0 then do;
      sumEd0=sumEd0 + (nodead0 + nodead1)*norisk0/(norisk0 + norisk1);
      sumEd1=sumEd1 + (nodead0 + nodead1)*norisk1/(norisk0 + norisk1);
      if (norisk0 + norisk1) > 1 then
         svar1d1=svar1d1 + (nodead0 + nodead1)*norisk0*norisk1*
                  (norisk0 + norisk1 - nodead0 -nodead1)
                /((norisk0 + norisk1)**2*(norisk0 + norisk1 - 1));
      svar2d1=svar2d1 + (norisk0*norisk1*(nodead0 + nodead1))
             / (norisk0 + norisk1)**2;
   end;
   if lastobs then do;
      chisq1 = (sumd1 - sumEd1)**2/svar1d1;
      chisq2 = (sumd1 - sumEd1)**2/svar2d1;
      chisq3 = (sumd1 - sumEd1)**2 * (1/sumEd0 + 1/sumEd1);
      put sumd1= sumEd1= svar1d1= svar2d1=;
      put chisq1= chisq2= chisq3=;
      probchi1 = 1 - probchi(chisq1,1);
      probchi2 = 1 - probchi(chisq2,1);
      probchi3 = 1 - probchi(chisq3,1);
      put probchi1= probchi2= probchi3=;
      end;
proc print;
proc plot;
   plot Shat0*time='0' lci0*time='-' uci0*time='-'
   Shat1*time='1' lci1*time='+' uci1*time='+'
   / overlay;
run;
quit;
