*-------------------------------------------------------------*
|  Survival data from Rio Grande wild turkeys monitored in    |
|  1986-87 along the South Platte River in northeast Colorado |
|                                                             |
|   captured = beginning of sampling period (1 week after     |
|              attachment of transmitters)                    |
|   returned = end of sampling period (until death, radio-    |
|              detachment or failure, or 31 July)             |
|   duration = tagend minus tagstart                          |
|                                                             |
|  note:  the status of 1028 (1986) and 101 (1987) are not     |
|         certain.  These may be radio-failures from live     |
|         birds instead of deaths.                            |
|                                                             |
*-------------------------------------------------------------*;

proc format;
   value age 1='Yearling' 2='Adult';
   value status 1='Lived' 2='Died' 3='Censored';

title 'Survival of Rio Grande Turkeys along South Platte River, CO';
*  The following data step reads in the observed data.;
data turkey;

 informat captured returned date.;
 input id year age captured returned duration status;
 format captured returned date.;
 format age age.;
 format status status.;
 cards;
      121   86 1   18-Feb-86   31-Jul-86      163 1
      196   86 1   18-Feb-86   31-Jul-86      163 1
      239   86 1   18-Feb-86   31-Jul-86      163 1
      324   86 1   18-Feb-86   22-May-86       93 1
      483   86 1   18-Feb-86   17-Mar-86       27 1
      724   86 1   18-Feb-86   10-Mar-86       20 1
     1028   86 1   18-Feb-86   20-Jun-86      122 3
     1301   86 2   18-Feb-86   31-Jul-86      163 1
     1341   86 2   18-Feb-86   08-May-86       79 2
     1361   86 1   18-Feb-86   31-Jul-86      163 1
     1402   86 1   18-Feb-86   31-Jul-86      163 1
     1435   86 2   18-Feb-86   31-Jul-86      163 1
     1461   86 2   18-Feb-86   31-Jul-86      163 1
     1483   86 2   18-Feb-86   09-Apr-86       50 1
     1620   86 1   18-Feb-86   31-Jul-86      163 1
     1651   86 2   18-Feb-86   31-Jul-86      163 1
     1701   86 1   18-Feb-86   31-Jul-86      163 1
      101   87 1   01-Mar-87   15-Apr-87       45 3
      197   87 1   01-Mar-87   21-May-87       81 1
      238   87 1   01-Mar-87   03-Jul-87      124 2
      326   87 1   01-Mar-87   31-Jul-87      152 1
      482   87 1   01-Mar-87   20-Jun-87      111 2
      724   87 1   01-Mar-87   31-Jul-87      152 1
      815   87 1   01-Mar-87   31-Jul-87      152 1
      850   87 1   18-Feb-87   31-Jul-87      163 1
      869   87 1   01-Mar-87   31-Jul-87      152 1
      906   87 1   12-Feb-87   07-Jun-87      115 1
      936   87 2   01-Mar-87   31-Jul-87      152 1
      964   87 1   18-Feb-87   31-Jul-87      163 1
     1006   87 2   12-Feb-87   31-Jul-87      169 1
     1034   87 2   14-Feb-87   31-Jul-87      167 1
     1057   87 2   01-Mar-87   31-Jul-87      152 1
     1123   87 1   01-Mar-87   31-Jul-87      152 1
     1148   87 2   18-Feb-87   31-Jul-87      163 1
     1234   87 1   14-Feb-87   26-Jun-87      132 2
     1342   87 2   01-Mar-87   31-Jul-87      152 1
     1483   87 1   14-Feb-87   31-Jul-87      167 1
;
proc print;

* This DATA step converts the file from captured and returned
  on the same record, to separate records for each.
  All animals are processed.;
data temp;
   title2 'Comparison of 1986 to 1987';
   set turkey;
   keep year type day;
   type='Captured';
   * Convert date to number of days since start of study,
     which is taken as February 1.;
   day=captured-mdy(02,1,year);
   label day='Days since start of study'
         type='Type of record';
   output;
   if status < 2 then return;
   else if status > 2 then type='Censored';
   else type='Died';
   day=returned-mdy(02,1,year);
   output;
proc print;
proc sort; by day;

* This DATA step generates input for the Kaplan-Meier estimates
  and log-rank test calculations.;
data lrinput;
   retain norisk0 nodead0 nocensr0 noadded0 0 firstday;
   retain norisk1 nodead1 nocensr1 noadded1 0;
   array norisk{86:87} norisk0 norisk1;
   array nodead{86:87} nodead0 nodead1;
   array nocensr{86:87} nocensr0 nocensr1;
   array noadded{86:87} noadded0 noadded1;
   set temp; by day;
   if _N_ = 1 then firstday=day;
   if type='Captured' then noadded{year}=noadded{year}+1;
   else if type='Died' then nodead{year}=nodead{year}+1;
   else if type='Censored' then nocensr{year}=nocensr{year}+1;
   if last.day then do;
      time=day;
      keep time norisk0 nodead0 nocensr0 noadded0
                norisk1 nodead1 nocensr1 noadded1;
      if time > 0 then output;
      do i=86 to 87 by 1;
         norisk{i}=norisk{i}+noadded{i}-nodead{i}-nocensr{i};
         nodead{i}=0; nocensr{i}=0; noadded{i}=0;
      end;
   end;
proc print;

* This data step computes the Kaplan-Meier Estimator for each group,
  plus calculates the log-rank test between groups.;
data logrank;
   title2 'Kaplan-Meier Estimates for 1986 and 1987';
   retain sumd1 0 sumEd0 0 sumEd1 0 svar1d1 0 svar2d1 0;
   retain Shat0 Shat1 1 Grensum0 Grensum1 0;
   set lrinput end=lastobs;
   * Compute the Kaplan-Meier Product Limit Estimator for
     each group, so that plots can be generated;
   if norisk0 > 0 then
      Shat0=Shat0*(1 - nodead0/norisk0);
   if norisk1 > 0 then
      Shat1=Shat1*(1 - nodead1/norisk1);
   if norisk0 > 0 & (norisk0 - nodead0) > 0 then
      Grensum0 = Grensum0 + nodead0/(norisk0*(norisk0-nodead0));
   if norisk1 > 0 & (norisk1 - nodead1) > 0 then
      Grensum1 = Grensum1 + nodead1/(norisk1*(norisk1-nodead1));
   seShat0 = sqrt(Shat0*Shat0*Grensum0);
   if seShat0 > 0 then do;
      lci0 = Shat0 - 1.96*seShat0;
      uci0 = Shat0 + 1.96*seShat0;
      end;
   else do;
      lci0=.;
      uci0=.;
      end;
   seShat1 = sqrt(Shat1*Shat1*Grensum1);
   if seShat1 > 0 then do;
      lci1 = Shat1 - 1.96*seShat1;
      uci1 = Shat1 + 1.96*seShat1;
      end;
   else do;
      lci1=.;
      uci1=.;
      end;
   keep time Shat0 Shat1 lci0 lci1 uci0 uci1;
   output;
   sumd1=sumd1+nodead1;
   if (norisk0 + norisk1) > 0 then do;
      sumEd0=sumEd0 + (nodead0 + nodead1)*norisk0/(norisk0 + norisk1);
      sumEd1=sumEd1 + (nodead0 + nodead1)*norisk1/(norisk0 + norisk1);
      if (norisk0 + norisk1) > 1 then
         svar1d1=svar1d1 + (nodead0 + nodead1)*norisk0*norisk1*
                  (norisk0 + norisk1 - nodead0 -nodead1)
                  /((norisk0 + norisk1)**2*(norisk0 + norisk1 - 1));
         svar2d1=svar2d1 + (norisk0*norisk1*(nodead0 + nodead1))
                  / (norisk0 + norisk1)**2;
   end;
   if lastobs then do;
      * Put the logrank test results out in the LOG file listing.;
      chisq1 = (sumd1 - sumEd1)**2/svar1d1;
      chisq2 = (sumd1 - sumEd1)**2/svar2d1;
      chisq3 = (sumd1 - sumEd1)**2 * (1/sumEd0 + 1/sumEd1);
      put sumd1= sumEd1= svar1d1= svar2d1=;
      put chisq1= chisq2= chisq3=;
      probchi1 = 1 - probchi(chisq1,1);
      probchi2 = 1 - probchi(chisq2,1);
      probchi3 = 1 - probchi(chisq3,1);
      put probchi1= probchi2= probchi3=;
      end;
proc print;
proc plot;
   plot Shat0*time='0' Shat1*time='1'
   lci0*time='-' uci0*time='-' lci1*time='+' uci1*time='+'
   / overlay;
/*
* Generate plot with SAS/GRAPH procedures;
symbol1 c=gold i=join v=plus;
symbol2 c=gold i=join v=x;
symbol3 c=white i=join v=;
proc gplot;
   plot Shat0*time=1 Shat1*time=2
   lci0*time=3 uci0*time=3 lci1*time=3 uci1*time=3
   / overlay;
*/
run;
