*--------------------------------------------------------------*
|   SAS code to map the confidence ellipse shape of            |
|      the ML estimator of Lenth (1981).                       |
|                                                              |
|   Developed by: Gary C. White                                |
|                 Department of Fishery and Wildlife Biology   |
|                 Colorado State University                    |
|                 Fort Collins, CO                             |
|                 (303)491-6678                                |
*--------------------------------------------------------------*;
title 'Map of confidence ellipse shape for 2,3, and 4 tower systems';
data precis;
   keep truex truey x y;
   label ellarea='Size of confidence ellipse';
   array beta{6};
   array sinb{6};
   array cosb{6};
   array utm{2,6};
   array a{2,2};
   array f1{2};
   pi=4*atan(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1;
   *Convert azimuth standard deviation to kappa;
   A1=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A1)+(1-A1)**2*(0.48794-0.82905*A1-1.3915*A1**2)/A1);
   /* Open file for Lotus input */
   file 'mapellip.dat';
do ntower=4;
   * Set coordinates of towers ;
   if ntower = 2 then do;
      utm{1,1}=0.101;  utm{2,1}=1.007;
      utm{1,2}=0.899;  utm{2,2}=1.007;
      end;
   else if ntower = 3 then do;
      utm{1,1}=0.500;  utm{2,1}=0.175;
      utm{1,2}=0.817;  utm{2,2}=0.671;
      utm{1,3}=0.183;  utm{2,3}=0.671;
      end;
   else if ntower = 4 then do;
      utm{1,1}=0.156;  utm{2,1}=0.500;
      utm{1,2}=0.500;  utm{2,2}=0.844;
      utm{1,3}=0.844;  utm{2,3}=0.500;
      utm{1,4}=0.500;  utm{2,4}=0.156;
      end;
   do truex=0 to 1 by 0.25;
      do truey=0 to 1 by 0.25;
         put truex truey;
         do i=1 to ntower;
            xp=truex-utm{1,i}; yp=truey-utm{2,i}; link atan2;
            beta{i}=alpha;
            sinb{i}=sin(beta{i});
            cosb{i}=cos(beta{i});
            end;
         link trimle;
         a{1,1}=xvar; a{1,2}=xycov;
         a{2,1}=a{1,2}; a{2,2}=yvar; n=2;
         link minv;
         xvar=a{1,1}/kappa; yvar=a{2,2}/kappa; xycov=a{1,2}/kappa;
         R=sqrt((xvar+yvar)**2-4*(xvar*yvar-xycov**2));
         theta=atan(-2*xycov/(yvar-xvar-R));
         acoef=sqrt((yvar+xvar+R)*cinv(0.95,2)/2);
         bcoef=sqrt((yvar+xvar-R)*cinv(0.95,2)/2);
         cost=cos(theta);
         sint=sin(theta);
         do psi=0 to 360 by 12;
            psir=psi*pi/180;
            acosp=acoef*cos(psir);
            bcosp=bcoef*sin(psir);
            x=truex+acosp*cost-bcosp*sint;
            y=truey+acosp*sint+bcosp*cost;
            put x y;
            end;
         end;
      end;
   end;
return;
trimle:
   xvar=0; xycov=0; yvar=0;
   do i=1 to ntower;
      di=sqrt((truex-utm{1,i})**2+(truey-utm{2,i})**2);
      cs=(truex-utm{1,i})/(di**3);
      ss=(truey-utm{2,i})/(di**3);
      xvar=xvar+sinb{i}*ss;
      xycov=xycov+cosb{i}*ss+sinb{i}*cs;
      yvar=yvar+cosb{i}*cs;
      end;
   xycov=xycov*(-0.5);
   return;
atan2: if abs(xp) < 0.00001 then do;
           if yp > 0 then alpha=pi/2;
           else if yp < 0 then alpha=3*pi/3;
           end;
        else do;
           alpha=atan(yp/xp);
           if xp < 0 & yp < 0 then alpha=alpha-pi;
           else if xp < 0 & yp >= 0 then alpha=alpha+pi;
           if alpha < 0 then alpha=alpha+2*pi;
        end;
   return;
minv:
   a{1,1}=1./a{1,1};
   do i=1 to n-1;
      do j=1 to i;
         f=0;
         do k=1 to i;
            f=f+a{j,k}*a{k,i+1};
            end;
         f1{j}=-f;
         end;
      f2=a{i+1,i+1};
      do j=1 to i;
         f2=f2+a{j,i+1}*f1{j};
         end;
      if abs(f2/a{i+1,i+1}) < 1.e-5 then put 'Matrix Singular';
      f2=1./f2;
      a{i+1,i+1}=f2;
      do j=1 to i;
         do k=1 to i;
            a{j,k}=f1{j}*f1{k}*f2+a{j,k};
            end;
         end;
      do j=1 to i;
         a{j,i+1}=f1{j}*f2;
         a{i+1,j}=a{j,i+1};
         end;
   end;
   return;
run;
