*-----------------------------------------------------------------*
|   SAS program to plot a map of typical the error polygons       |
|      for a 2 tower triangulation system where the locations     |
|      of the two towers (x1, y1, and x2, y2) minimize the        |
|      average error polygon size for a square study area.        |
|                                                                 |
|   Developed by: Gary C. White                                   |
|                 Department of Fishery and Wildlife Biology      |
|                 Colorado State University                       |
|                 Fort Collins, CO 80523                          |
|                 (303)491-6678                                   |
*-----------------------------------------------------------------*;
title 'Map of Error Polygon Size of 2 Tower Triangulation System';
title2 'Towers located to minimize average polygon size';
data triang2;
   array beta{2} beta1 beta2;
   array poly{2,4} xa xb xc xd ya yb yc yd;
   keep x1 y1 x2 y2 truex truey x y xa ya xb yb xc yc xd yd area;
   pi=4*atan(1);
   s=1*pi/180;
   /* Set tower locations */
   x1=0.101; y1=1.007; x2=0.899; y2=1.007;
   /* Open file for input to Lotus */
   file 'mappoly2.dat';
   do truex=0 to 1 by 0.25;
      do truey=0 to 0.75 by 0.25;
         put truex truey;
         yp=truey-y1; xp=truex-x1; link atan2; tbeta1=alpha;
         yp=truey-y2; xp=truex-x2; link atan2; tbeta2=alpha;
         beta1=tbeta1; beta2=tbeta2;
         link calcu;
         xl=x;yl=y;
         /* Calculate area of error polygon */
         beta1=tbeta1-1.96*s;   /* -- coordinate */
         beta2=tbeta2-1.96*s;
         link calcu; poly{1,1}=x; poly{2,1}=y;
         beta1=tbeta1+1.96*s;   /* ++ coordinate */
         beta2=tbeta2+1.96*s;
         link calcu; poly{1,2}=x; poly{2,2}=y;
         beta1=tbeta1-1.96*s;   /* -+ coordinate */
         link calcu; poly{1,3}=x; poly{2,3}=y;
         /* Get area of half of error polygon on -+ side */
         d1=sqrt((poly{1,1}-poly{1,2})**2+(poly{2,1}-poly{2,2})**2);
         d2=sqrt((poly{1,2}-poly{1,3})**2+(poly{2,2}-poly{2,3})**2);
         d3=sqrt((poly{1,1}-poly{1,3})**2+(poly{2,1}-poly{2,3})**2);
         sp=(d1+d2+d3)/2;
         area=sqrt(sp*(sp-d1)*(sp-d2)*(sp-d3));
         beta1=tbeta1+1.96*s;   /* +- coordinate */
         beta2=tbeta2-1.96*s;
         link calcu; poly{1,4}=x; poly{2,4}=y;
         /* Now get area on other side of polygon */
         d2=sqrt((poly{1,2}-poly{1,4})**2+(poly{2,2}-poly{2,4})**2);
         d3=sqrt((poly{1,1}-poly{1,4})**2+(poly{2,1}-poly{2,4})**2);
         sp=(d1+d2+d3)/2;
         area=area+sqrt(sp*(sp-d1)*(sp-d2)*(sp-d3));
         put xa ya;
         put xd yd;
         put xb yb;
         put xc yc;
         put xa ya;
         output;
      end;
   end;
   return;
atan2:  alpha=atan(yp/xp);
        if xp < 0 & yp < 0 then alpha=alpha-pi;
        else if xp < 0 & yp >= 0 then alpha=alpha+pi;
        if alpha < 0 then alpha=alpha+2*pi;
   return;
calcu:
   x=(x1*tan(beta1)-x2*tan(beta2)+y2-y1)/(tan(beta1)-tan(beta2));
   y=((x2-x1)*tan(beta1)*tan(beta2)-y2*tan(beta1)+y1*tan(beta2))
      /(tan(beta2)-tan(beta1));
   return;
proc print; run;
