*--------------------------------------------------------------*
|   SAS code to calculate the MLE of Lenth (1981).             |
*--------------------------------------------------------------*;
data trimle;
   drop pi A kappa v1 v2 v3 i j noiter converge initial
        zi di ss cs xm ym azcalc;
   label xl='MLE of x coordinate'
         yl='MLE of y coordinate'
         s='Standard deviation of azimuths (degrees)'
         ellarea='Size of confidence ellipse'
         itower1='Tower number of azimuth 1'
         az1='Azimuth 1'
         itower2='Tower number of azimuth 2'
         az2='Azimuth 2'
         itower3='Tower number of azimuth 3'
         az3='Azimuth 3'
         date='Date of location'
         time='Time of location'
         id='Animal identification'
         modulat='Modulating or steady signal';
   array itower{3} itower1 itower2 itower3;
   array az{3} az1 az2 az3;
   array trueaz{3};
   array beta{3};
   array sinb{3};
   array cosb{3};
   array utm{2,8};
   array amat{2,2};
   array bmat{2};
   pi=4*ATAN(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1.5;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=746917;  utm{2,1}=4391247;
   utm{1,2}=748353;  utm{2,2}=4390596;
   utm{1,3}=748212;  utm{2,3}=4392147;
   utm{1,4}=749953;  utm{2,4}=4391223;
   utm{1,5}=751023;  utm{2,5}=4392418;
   utm{1,6}=750142;  utm{2,6}=4393673;
   utm{1,7}=751674;  utm{2,7}=4393630;
   utm{1,8}=750985;  utm{2,8}=4395022;
   /* Position animal at middle of 3 towers */
   xtrue=mean(utm{1,1},utm{1,2},utm{1,3});
   ytrue=mean(utm{2,1},utm{2,2},utm{2,3});
   do i=1 to dim(az);
      itower{i}=i;
      /* SAS code to calculate bearing from tower
         to true location */
      xm=xtrue-utm{1,itower{i}};
      ym=ytrue-utm{2,itower{i}};
      azcalc=atan(ym/xm);
      if xm < 0 & ym < 0 then azcalc=azcalc-pi;
      else if xm < 0 & ym >= 0 then azcalc=azcalc+pi;
      if azcalc < 0 then azcalc=azcalc+2*pi;
      /* Now convert angle in radians to bearing in degrees */
      azcalc=90-azcalc*(180/pi);
      if azcalc < 0 then azcalc=azcalc+360;
      if azcalc > 360 then azcalc=azcalc-360;
      trueaz{i}=azcalc;
      end;
   seed=0;
   do irep=1 to 3000;
      /* Tack on a random error on true bearing */
      do i=1 to dim(az);
         az{i}=trueaz{i}+rannor(seed)*s;
         end;
   *Code to summarize azimuths and plot;
   do i=1 to dim(az);
      /* Convert bearing or azimuth to an angle */
      beta{i}=(90-az{i})*(pi/180);
      sinb{i}=sin(beta{i});
      cosb{i}=cos(beta{i});
   end;
   noiter=0;
   *Obtain initial estimate;
   xl=0; yl=0;
   do i=1 to dim(az);
      xl=xl+utm{1,itower{i}};
      yl=yl+utm{2,itower{i}};
      end;
   xl=xl/dim(az); yl=yl/dim(az);
   converge=0;
   initial=1;
iter:
   do i=1 to 2;
      bmat{i}=0;
      do j=1 to 2;
         amat{i,j}=0;
         end;
      end;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         zi=sinb{i}*utm{1,itower{i}}-cosb{i}*utm{2,itower{i}};
         if initial then do;
            ss=sinb{i};
            cs=cosb{i};
            end;
         di=sqrt((xl-utm{1,itower{i}})**2+(yl-utm{2,itower{i}})**2);
         ss=(yl-utm{2,itower{i}})/(di**3);
         cs=(xl-utm{1,itower{i}})/(di**3);
         amat{1,1}=amat{1,1}+sinb{i}*ss;
         amat{2,2}=amat{2,2}+cosb{i}*cs;
         amat{1,2}=amat{1,2}-cosb{i}*ss;
         amat{2,1}=amat{2,1}-sinb{i}*cs;
         bmat{1}=bmat{1}+ss*zi;
         bmat{2}=bmat{2}-cs*zi;
         end;
      end;
   ym=(amat{1,1}*bmat{2}-amat{2,1}*bmat{1})
       /(amat{1,1}*amat{2,2}-amat{2,1}*amat{1,2});
   xm=(bmat{1}-amat{1,2}*ym)/amat{1,1};
   initial=0;
   if abs(xm-xl)/xm < 0.00001 and abs(ym-yl)/ym < 0.00001 then converge=1;
   xl=xm; yl=ym;
   noiter=noiter+1;
   if converge = 0 and noiter < 40 then goto iter;
   if noiter >= 40 then do;
      put 'ERROR -- Number of iterations exceed limit';
      put '   ' _N_= itower1= az1= itower2= az2= itower3= az3= ;
      end;
   *Convergence achieved - get vc mat;
   v1=0; v2=0; v3=0;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         di=sqrt((xl-utm{1,itower{i}})**2+(yl-utm{2,itower{i}})**2);
         cs=(xl-utm{1,itower{i}})/(di**3);
         ss=(yl-utm{2,itower{i}})/(di**3);
         v1=v1+sinb{i}*ss;
         v2=v2+cosb{i}*ss+sinb{i}*cs;
         v3=v3+cosb{i}*cs;
         end;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(1./((v1*v3-v2*v2)*kappa**2));
   *Construct chi-square goodness of fit test;
   chisq=0.;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         /* SAS code to calculate bearing from tower
            to estimated location */
         xm=xl-utm{1,itower{i}};
         ym=yl-utm{2,itower{i}};
         azcalc=atan(ym/xm);
         if xm < 0 & ym < 0 then azcalc=azcalc-pi;
         else if xm < 0 & ym >= 0 then azcalc=azcalc+pi;
         if azcalc < 0 then azcalc=azcalc+2*pi;
         /* Now convert angle in radians to bearing in degrees */
         azcalc=90-azcalc*(180/pi);
         if azcalc < 0 then azcalc=azcalc+360;
         if azcalc > 360 then azcalc=azcalc-360;
         /* Now determine difference between observed and
            expected bearings */
         xm=min(az{i},azcalc);
         ym=max(az{i},azcalc);
         if xm < 90 & ym > 270 then xm=xm+360;
         chisq=chisq+((xm-ym)/s)**2;
         end;
      else chisq=.;
      end;
   gofprob=1-probchi(chisq,dim(az)-2);
   gofprob=int(gofprob*20)/20;
   keep gofprob;
   output;
   end;
proc freq; tables gofprob; run;
proc chart; vbar gofprob / midpoints=0 to 1 by 0.05; run;
