*--------------------------------------------------------------*
|   SAS code to map the confidence area ellipse size of        |
|      the ML estimator of Lenth (1981).                       |
|                                                              |
|   Developed by: Gary C. White                                |
|                 Department of Fishery and Wildlife Biology   |
|                 Colorado State University                    |
|                 Fort Collins, CO                             |
|                 (303)491-6678                                |
*--------------------------------------------------------------*;
title 'Map of confidence ellipse size for 2-tower system';
data precis;
   keep truex truey ellarea;
   label ellarea='Size of confidence ellipse';
   array beta{2};
   array sinb{2};
   array cosb{2};
   array utm{2,2};
   pi=4*atan(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=0.101;  utm{2,1}=1.007;
   utm{1,2}=0.899;  utm{2,2}=1.007;
   do truex=0 to 1 by 1/33;
      do truey=0 to 1 by 1/33;
         do i=1 to dim(beta);
            xp=truex-utm{1,i}; yp=truey-utm{2,i}; link atan2;
            beta{i}=alpha;
            sinb{i}=sin(beta{i});
            cosb{i}=cos(beta{i});
            end;
         link trimle;
         output;
         end;
      end;
   return;
trimle:
   v1=0; v2=0; v3=0;
   do i=1 to dim(beta);
      di=sqrt((truex-utm{1,i})**2+(truey-utm{2,i})**2);
      cs=(truex-utm{1,i})/(di**3);
      ss=(truey-utm{2,i})/(di**3);
      v1=v1+sinb{i}*ss;
      v2=v2+cosb{i}*ss+sinb{i}*cs;
      v3=v3+cosb{i}*cs;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(max(0,1/((v1*v3-v2*v2)*kappa**2)));
   return;
atan2: if abs(xp) < 0.00001 then do;
           if yp > 0 then alpha=pi/2;
           else if yp < 0 then alpha=3*pi/3;
           end;
        else do;
           alpha=atan(yp/xp);
           if xp < 0 & yp < 0 then alpha=alpha-pi;
           else if xp < 0 & yp >= 0 then alpha=alpha+pi;
           if alpha < 0 then alpha=alpha+2*pi;
        end;
   return;
proc means maxdec=8; var ellarea;
proc plot; plot truey*truex=ellarea / contour=10;
   title2 'Contours of error polygon size';
run;
*--------------------------------------------------------------*
|   SAS code to map the confidence area ellipse size of        |
|      the ML estimator of Lenth (1981).                       |
|                                                              |
|   Developed by: Gary C. White                                |
|                 Department of Fishery and Wildlife Biology   |
|                 Colorado State University                    |
|                 Fort Collins, CO                             |
|                 (303)491-6678                                |
*--------------------------------------------------------------*;
title 'Map of confidence ellipse size for 3-tower system';
data precis;
   keep truex truey ellarea;
   label ellarea='Size of confidence ellipse';
   array beta{3};
   array sinb{3};
   array cosb{3};
   array utm{2,3};
   pi=4*atan(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=0.500;  utm{2,1}=0.175;
   utm{1,2}=0.817;  utm{2,2}=0.671;
   utm{1,3}=0.183;  utm{2,3}=0.671;
   do truex=0 to 1 by 1/33;
      do truey=0 to 1 by 1/33;
         do i=1 to dim(beta);
            xp=truex-utm{1,i}; yp=truey-utm{2,i}; link atan2;
            beta{i}=alpha;
            sinb{i}=sin(beta{i});
            cosb{i}=cos(beta{i});
            end;
         link trimle;
         output;
         end;
      end;
   return;
trimle:
   v1=0; v2=0; v3=0;
   do i=1 to dim(beta);
      di=sqrt((truex-utm{1,i})**2+(truey-utm{2,i})**2);
      cs=(truex-utm{1,i})/(di**3);
      ss=(truey-utm{2,i})/(di**3);
      v1=v1+sinb{i}*ss;
      v2=v2+cosb{i}*ss+sinb{i}*cs;
      v3=v3+cosb{i}*cs;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(max(0,1/((v1*v3-v2*v2)*kappa**2)));
   return;
atan2: if abs(xp) < 0.00001 then do;
           if yp > 0 then alpha=pi/2;
           else if yp < 0 then alpha=3*pi/3;
           end;
        else do;
           alpha=atan(yp/xp);
           if xp < 0 & yp < 0 then alpha=alpha-pi;
           else if xp < 0 & yp >= 0 then alpha=alpha+pi;
           if alpha < 0 then alpha=alpha+2*pi;
        end;
   return;
proc means maxdec=8; var ellarea;
proc plot; plot truey*truex=ellarea / contour=10;
   title2 'Contours of error polygon size';
run;
*--------------------------------------------------------------*
|   SAS code to map the confidence area ellipse size of        |
|      the ML estimator of Lenth (1981).                       |
|                                                              |
|   Developed by: Gary C. White                                |
|                 Department of Fishery and Wildlife Biology   |
|                 Colorado State University                    |
|                 Fort Collins, CO                             |
|                 (303)491-6678                                |
*--------------------------------------------------------------*;
title 'Map of confidence ellipse size for 4-tower system';
data precis;
   keep truex truey ellarea;
   label ellarea='Size of confidence ellipse';
   array beta{4};
   array sinb{4};
   array cosb{4};
   array utm{2,4};
   pi=4*atan(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=0.156;  utm{2,1}=0.500;
   utm{1,2}=0.500;  utm{2,2}=0.844;
   utm{1,3}=0.844;  utm{2,3}=0.500;
   utm{1,4}=0.500;  utm{2,4}=0.156;
   do truex=0 to 1 by 1/33;
      do truey=0 to 1 by 1/33;
         do i=1 to dim(beta);
            xp=truex-utm{1,i}; yp=truey-utm{2,i}; link atan2;
            beta{i}=alpha;
            sinb{i}=sin(beta{i});
            cosb{i}=cos(beta{i});
            end;
         link trimle;
         output;
         end;
      end;
   return;
trimle:
   v1=0; v2=0; v3=0;
   do i=1 to dim(beta);
      di=sqrt((truex-utm{1,i})**2+(truey-utm{2,i})**2);
      cs=(truex-utm{1,i})/(di**3);
      ss=(truey-utm{2,i})/(di**3);
      v1=v1+sinb{i}*ss;
      v2=v2+cosb{i}*ss+sinb{i}*cs;
      v3=v3+cosb{i}*cs;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(max(0,1/((v1*v3-v2*v2)*kappa**2)));
   return;
atan2: if abs(xp) < 0.00001 then do;
           if yp > 0 then alpha=pi/2;
           else if yp < 0 then alpha=3*pi/3;
           end;
        else do;
           alpha=atan(yp/xp);
           if xp < 0 & yp < 0 then alpha=alpha-pi;
           else if xp < 0 & yp >= 0 then alpha=alpha+pi;
           if alpha < 0 then alpha=alpha+2*pi;
        end;
   return;
proc means maxdec=8; var ellarea;
proc plot; plot truey*truex=ellarea / contour=10;
   title2 'Contours of error polygon size';
run;
*--------------------------------------------------------------*
|   SAS code to map the confidence area ellipse size of        |
|      the ML estimator of Lenth (1981).                       |
|                                                              |
|   Developed by: Gary C. White                                |
|                 Department of Fishery and Wildlife Biology   |
|                 Colorado State University                    |
|                 Fort Collins, CO                             |
|                 (303)491-6678                                |
*--------------------------------------------------------------*;
title 'Map of confidence ellipse size for 5-tower system';
data precis;
   keep truex truey ellarea;
   label ellarea='Size of confidence ellipse';
   array beta{5};
   array sinb{5};
   array cosb{5};
   array utm{2,5};
   pi=4*atan(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=0.156;  utm{2,1}=0.454;
   utm{1,2}=0.336;  utm{2,2}=0.839;
   utm{1,3}=0.834;  utm{2,3}=0.319;
   utm{1,4}=0.432;  utm{2,4}=0.154;
   utm{1,5}=0.781;  utm{2,5}=0.765;
   do truex=0 to 1 by 1/33;
      do truey=0 to 1 by 1/33;
         do i=1 to dim(beta);
            xp=truex-utm{1,i}; yp=truey-utm{2,i}; link atan2;
            beta{i}=alpha;
            sinb{i}=sin(beta{i});
            cosb{i}=cos(beta{i});
            end;
         link trimle;
         output;
         end;
      end;
   return;
trimle:
   v1=0; v2=0; v3=0;
   do i=1 to dim(beta);
      di=sqrt((truex-utm{1,i})**2+(truey-utm{2,i})**2);
      cs=(truex-utm{1,i})/(di**3);
      ss=(truey-utm{2,i})/(di**3);
      v1=v1+sinb{i}*ss;
      v2=v2+cosb{i}*ss+sinb{i}*cs;
      v3=v3+cosb{i}*cs;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(max(0,1/((v1*v3-v2*v2)*kappa**2)));
   return;
atan2: if abs(xp) < 0.00001 then do;
           if yp > 0 then alpha=pi/2;
           else if yp < 0 then alpha=3*pi/3;
           end;
        else do;
           alpha=atan(yp/xp);
           if xp < 0 & yp < 0 then alpha=alpha-pi;
           else if xp < 0 & yp >= 0 then alpha=alpha+pi;
           if alpha < 0 then alpha=alpha+2*pi;
        end;
   return;
proc means maxdec=8; var ellarea;
proc plot; plot truey*truex=ellarea / contour=10;
   title2 'Contours of error polygon size';
run;
