*-----------------------------------------------------------------*
|   SAS program to estimate the location (xl, yl) of a            |
|      radio, given the locations of two towers (x1, y1, and      |
|      x2, y2) and the bearings from the towers to the radio      |
|      (alpha1, alpha2).                                          |
|                                                                 |
|   Developed by: Gary C. White                                   |
|                 Department of Fishery and Wildlife Biology      |
|                 Colorado State University                       |
|                 Fort Collins, CO 80523                          |
|                 (303)491-6678                                   |
*-----------------------------------------------------------------*;
data triang2;
   array alpha{2} alpha1 alpha2;
   array beta{2} beta1 beta2;
   array poly{2,4};
   drop i beta1 beta2 pi x y ok d1 d2 d3 sp;
   pi=4*atan(1);
   s=1.5;
   input x1 y1 x2 y2 alpha1 alpha2;
   do i=1 to 2;
      beta{i}=(90-alpha{i})*(pi/180);
      end;
   link calcu;
   xl=x;yl=y;
   if ok then do;
      /* If estimate is on top of one of the towers,
         the area is zero. */
      if (abs(xl/x1-1)<0.0001 & abs(yl/y1-1)<0.0001)
         | (abs(xl/x2-1)<0.0001 & abs(yl/y2-1)<0.0001) then area=0;
      else do;
         /* Calculate area of error polygon */
         beta1=(90-(alpha1-1.96*s))*(pi/180);   /* -- coordinate */
         beta2=(90-(alpha2-1.96*s))*(pi/180);
         link calcu; poly{1,1}=x; poly{2,1}=y;
         beta1=(90-(alpha1+1.96*s))*(pi/180);   /* ++ coordinate */
         beta2=(90-(alpha2+1.96*s))*(pi/180);
         link calcu; poly{1,2}=x; poly{2,2}=y;
         beta1=(90-(alpha1-1.96*s))*(pi/180);   /* -+ coordinate */
         link calcu; poly{1,3}=x; poly{2,3}=y;
         /* Get area of half of error polygon on -+ side */
         d1=sqrt((poly{1,1}-poly{1,2})**2+(poly{2,1}-poly{2,2})**2);
         d2=sqrt((poly{1,2}-poly{1,3})**2+(poly{2,2}-poly{2,3})**2);
         d3=sqrt((poly{1,1}-poly{1,3})**2+(poly{2,1}-poly{2,3})**2);
         sp=(d1+d2+d3)/2;
         area=sqrt(sp*(sp-d1)*(sp-d2)*(sp-d3));
         beta1=(90-(alpha1+1.96*s))*(pi/180);   /* +- coordinate */
         beta2=(90-(alpha2-1.96*s))*(pi/180);
         link calcu; poly{1,4}=x; poly{2,4}=y;
         /* Now get area on other side of polygon */
         d2=sqrt((poly{1,2}-poly{1,4})**2+(poly{2,2}-poly{2,4})**2);
         d3=sqrt((poly{1,1}-poly{1,4})**2+(poly{2,1}-poly{2,4})**2);
         sp=(d1+d2+d3)/2;
         area=area+sqrt(sp*(sp-d1)*(sp-d2)*(sp-d3));
      end;
   end;
   else area=.;
   return;
calcu:
   if abs(beta1-beta2) < pi/90
      | abs(abs(beta1-beta2) - pi) < pi/90 then do;
      x=.; y=.; ok=0; end;
   else do;
      x=(x1*tan(beta1)-x2*tan(beta2)+y2-y1)/(tan(beta1)-tan(beta2));
      y=((x2-x1)*tan(beta1)*tan(beta2)-y2*tan(beta1)+y1*tan(beta2))
         /(tan(beta2)-tan(beta1));
      ok=1;
      end;
   return;
   cards;
 1 1 2 1  45 315
 1 1 2 1   0 315
 1 1 2 1 359 315
 1 1 2 1 180 225
 1 1 2 1 179 225
 1 1 2 1 181 225
 1 1 2 1 135 180
 1 1 2 1 135 179
 1 1 2 1 135 181
 1 1 2 1  45   0
 1 1 2 1  45   1
 1 1 2 1  45 359
 1 1 2 1  90   1
 1 1 2 1   0   0
 1 1 2 1  90  90
 1 1 2 1  90 270
 1 1 2 1  45 225
 1 1 2 1 315 135
 1 1 2 1   0 180
 1 1 2 2   0 270
 1 1 2 2   0 315
 1 1 2 2  45 225
 1 1 2 2 315 270
 1 1 2 2  45  45
 1 1 2 2 315  45
 1 1 2 2 315 225
 1 1 2 2 195 210
;
proc print;
run;
