*--------------------------------------------------------------*
|   SAS code to calculate the MLE of Lenth (1981).             |
*--------------------------------------------------------------*;
data trimle;
   drop pi A kappa v1 v2 v3 i j noiter converge initial
        zi di ss cs xm ym azcalc;
   label xl='MLE of x coordinate'
         yl='MLE of y coordinate'
         s='Standard deviation of azimuths (degrees)'
         ellarea='Size of confidence ellipse'
         itower1='Tower number of azimuth 1'
         az1='Azimuth 1'
         itower2='Tower number of azimuth 2'
         az2='Azimuth 2'
         itower3='Tower number of azimuth 3'
         az3='Azimuth 3'
         date='Date of location'
         time='Time of location'
         id='Animal identification'
         modulat='Modulating or steady signal';
   array itower{3} itower1 itower2 itower3;
   array az{3} az1 az2 az3;
   array beta{3};
   array sinb{3};
   array cosb{3};
   array utm{2,8};
   array amat{2,2};
   array bmat{2};
   pi=4*ATAN(1);
   * Set Azimuth Standard Deviation (in degrees) here;
   s=1.5;
   *Convert azimuth standard deviation to kappa;
   A=exp((s*pi/180)**2*(-0.5));
   kappa=1/(2*(1-A)+(1-A)**2*(0.48794-0.82905*A-1.3915*A**2)/A);
   * Set coordinates of towers ;
   utm{1,1}=746917;  utm{2,1}=4391247;
   utm{1,2}=748353;  utm{2,2}=4390596;
   utm{1,3}=748212;  utm{2,3}=4392147;
   utm{1,4}=749953;  utm{2,4}=4391223;
   utm{1,5}=751023;  utm{2,5}=4392418;
   utm{1,6}=750142;  utm{2,6}=4393673;
   utm{1,7}=751674;  utm{2,7}=4393630;
   utm{1,8}=750985;  utm{2,8}=4395022;
   informat date mmddyy8. time time8.;
   *Modify this statement for changes in number of towers;
   input date time id $ modulat $
      itower1 az1 itower2 az2 itower3 az3;
   format date date. time time.;
   *Code to summarize azimuths and plot;
   do i=1 to dim(az);
      /* Convert bearing or azimuth to an angle */
      beta{i}=(90-az{i})*(pi/180);
      sinb{i}=sin(beta{i});
      cosb{i}=cos(beta{i});
   end;
   noiter=0;
   *Obtain initial estimate;
   xl=0; yl=0;
   do i=1 to dim(az);
      xl=xl+utm{1,itower{i}};
      yl=yl+utm{2,itower{i}};
      end;
   xl=xl/dim(az); yl=yl/dim(az);
   converge=0;
   initial=1;
iter:
   do i=1 to 2;
      bmat{i}=0;
      do j=1 to 2;
         amat{i,j}=0;
         end;
      end;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         zi=sinb{i}*utm{1,itower{i}}-cosb{i}*utm{2,itower{i}};
         if initial then do;
            ss=sinb{i};
            cs=cosb{i};
            end;
         di=sqrt((xl-utm{1,itower{i}})**2+(yl-utm{2,itower{i}})**2);
         ss=(yl-utm{2,itower{i}})/(di**3);
         cs=(xl-utm{1,itower{i}})/(di**3);
         amat{1,1}=amat{1,1}+sinb{i}*ss;
         amat{2,2}=amat{2,2}+cosb{i}*cs;
         amat{1,2}=amat{1,2}-cosb{i}*ss;
         amat{2,1}=amat{2,1}-sinb{i}*cs;
         bmat{1}=bmat{1}+ss*zi;
         bmat{2}=bmat{2}-cs*zi;
         end;
      end;
   ym=(amat{1,1}*bmat{2}-amat{2,1}*bmat{1})
       /(amat{1,1}*amat{2,2}-amat{2,1}*amat{1,2});
   xm=(bmat{1}-amat{1,2}*ym)/amat{1,1};
   initial=0;
   if abs(xm-xl)/xm < 0.00001 and abs(ym-yl)/ym < 0.00001 then converge=1;
   xl=xm; yl=ym;
   noiter=noiter+1;
   if converge = 0 and noiter < 40 then goto iter;
   if noiter >= 40 then do;
      put 'ERROR -- Number of iterations exceed limit';
      put '   ' _N_= itower1= az1= itower2= az2= itower3= az3= ;
      end;
   *Convergence achieved - get vc mat;
   v1=0; v2=0; v3=0;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         di=sqrt((xl-utm{1,itower{i}})**2+(yl-utm{2,itower{i}})**2);
         cs=(xl-utm{1,itower{i}})/(di**3);
         ss=(yl-utm{2,itower{i}})/(di**3);
         v1=v1+sinb{i}*ss;
         v2=v2+cosb{i}*ss+sinb{i}*cs;
         v3=v3+cosb{i}*cs;
         end;
      end;
   v2=v2*(-0.5);
   ellarea=pi*cinv(0.95,2)*sqrt(1./((v1*v3-v2*v2)*kappa**2));
   *Construct chi-square goodness of fit test;
   chisq=0.;
   do i=1 to dim(az);
      if az{i} ^= . then do;
         /* SAS code to calculate bearing from tower
            to estimated location */
         xm=xl-utm{1,itower{i}};
         ym=yl-utm{2,itower{i}};
         azcalc=atan(ym/xm);
         if xm < 0 & ym < 0 then azcalc=azcalc-pi;
         else if xm < 0 & ym >= 0 then azcalc=azcalc+pi;
         if azcalc < 0 then azcalc=azcalc+2*pi;
         /* Now convert angle in radians to bearing in degrees */
         azcalc=90-azcalc*(180/pi);
         if azcalc < 0 then azcalc=azcalc+360;
         if azcalc > 360 then azcalc=azcalc-360;
         /* Now determine difference between observed and
            expected bearings */
         xm=min(az{i},azcalc);
         ym=max(az{i},azcalc);
         if xm < 90 & ym > 270 then xm=xm+360;
         chisq=chisq+((xm-ym)/s)**2;
         put az{i}= azcalc= xm= ym= chisq=;
         end;
      else chisq=.;
      end;
   gofprob=1-probchi(chisq,dim(az)-2);
cards;
06/06/84   14:38:43   210 S  1 103 2 297 3 229
06/06/84   15:04:30   210 S  1 101 2 298 3 233
06/06/84   15:34:32   210 S  1 102 2 298 3 229
06/06/84   17:07:31   210 M  1  99 2 300 3 229
06/06/84   17:40:48   210 M  1 165 2 281 3   .
06/06/84   18:07:51   210 S  1 150 2 284 3 221
06/06/84   18:38:40   210 S  1 146 2 284 3 222
06/06/84   20:40:05   210 M  1 153 2 284 3 219
06/06/84   21:16:57   210 M  1 154 2 287 3 217
06/06/84   21:42:30   210 S  1 106 2 290 3 216
06/06/84   22:06:30   210 S  1 148 2 289 3 216
06/06/84   23:08:06   210 S  1  95 2 290 3 215
06/09/84   00:17:50   210 M  1 165 2 286 3 227
06/09/84   00:40:59   210 M  1 144 2 287 3 227
06/09/84   01:07:28   210 S  1 151 2 284 3 223
06/09/84   01:35:54   210 M  1 157 2 280 3 222
06/09/84   02:03:52   210 S  1 179 2 276 3 224
06/09/84   02:33:55   210 M  1 179 2 277 3 224
06/09/84   03:04:18   210 S  1 180 2 277 3 224
06/09/84   03:32:25   210 S  1 180 2 276 3 224
06/09/84   04:08:17   210 S  1 182 2 276 3 224
06/09/84   04:32:52   210 S  1 181 2 278 3 224
06/09/84   05:05:23   210 M  1 181 2 277 3 224
06/09/84   09:37:04   210 S  1 348 2 302 3 241
06/09/84   10:06:24   210 S  1  20 2 305 3 243
06/09/84   10:34:59   210 S  1  17 2 304 3 243
06/09/84   11:02:40   210 S  1  24 2 305 3 243
06/09/84   11:31:56   210 M  1  14 2 305 3 243
06/23/84   12:09:48   210 M  1 341 2 299 3 241
06/23/84   12:34:24   210 M  1 349 2 300 3 241
06/23/84   13:03:19   210 S  1 346 2 300 3 241
06/23/84   13:33:41   210 S  1 346 2 298 3 241
06/23/84   14:03:19   210 S  1 343 2 301 3 242
06/23/84   14:33:53   210 S  1 348 2 301 3 241
06/23/84   15:06:28   210 S  1 349 2 300 3 241
06/23/84   15:37:27   210 S  1 340 2 302 3 241
06/23/84   16:12:15   210 S  1 340 2 301 3 243
06/23/84   16:32:53   210 S  1 354 2 303 3 244
06/23/84   17:04:02   210 S  1 343 2 307 3 244
06/23/84   17:32:47   210 S  1 342 2 308 3 243
06/23/84   18:06:38   210 S  1 342 2 309 3 243
06/23/84   18:35:07   210 S  1   . 2 308 3 241
06/23/84   19:03:55   210 M  1  46 2 306 3 238
06/23/84   19:33:23   210 M  1   . 2 302 3 232
06/23/84   20:04:04   210 S  1   . 2 300 3 234
06/23/84   20:34:55   210 M  1   . 2 336 3 233
06/23/84   21:06:20   210 M  1  40 2 332 3 223
06/23/84   21:33:28   210 M  1  90 2 326 3 222
06/23/84   22:02:39   210 S  1   . 2 326 3 219
06/23/84   22:34:07   210 S  1 160 2 291 3 220
06/23/84   23:03:12   210 S  1 155 2 292 3 220
06/23/84   23:28:23   210 S  1 152 2 292 3 220
06/26/84   00:22:01   210 S  1  95 2 291 3 222
06/26/84   00:36:37   210 S  1  99 2 292 3 220
06/26/84   01:06:29   210 S  1   . 2 283 3 220
06/26/84   01:36:29   210 S  1 161 2 282 3 221
06/26/84   02:04:30   210 S  1 157 2 282 3 222
06/26/84   02:35:37   210 S  1 158 2 282 3 222
06/26/84   03:04:52   210 S  1 159 2 282 3 212
06/26/84   03:34:09   210 S  1 159 2 282 3 221
06/26/84   04:08:26   210 S  1 159 2 282 3 223
06/26/84   04:34:49   210 S  1 160 2 282 3 222
06/26/84   05:05:02   210 M  1 153 2 284 3 221
06/26/84   05:35:48   210 M  1 147 2 289 3 227
06/26/84   06:05:45   210 M  1   . 2 297 3 237
06/26/84   06:36:56   210 M  1   . 2 303 3 240
06/26/84   07:05:17   210 S  1  23 2 303 3 241
06/26/84   07:40:02   210 S  1  24 2   . 3 238
06/26/84   08:09:43   210 M  1  24 2   . 3 241
06/26/84   08:37:57   210 M  1   2 2   . 3 240
06/26/84   09:06:28   210 S  1  10 2   . 3 243
06/26/84   09:36:09   210 S  1  10 2   . 3 242
06/26/84   10:04:00   210 S  1   9 2   . 3 242
06/26/84   10:36:11   210 S  1   9 2   . 3 243
06/26/84   11:04:08   210 S  1   8 2   . 3 242
06/26/84   11:35:53   210 S  1   8 2   . 3 242
07/09/84   12:16:04   210 S  1  32 2   . 3 243
07/09/84   12:39:35   210 S  1  31 2 307 3 243
07/09/84   13:02:53   210 S  1  31 2 308 3 243
07/09/84   13:33:08   210 S  1  15 2 306 3 246
07/09/84   14:02:38   210 M  1  13 2 306 3 244
07/09/84   14:33:32   210 S  1  18 2 308 3 243
07/09/84   15:08:02   210 M  1  15 2 307 3 246
07/09/84   15:33:34   210 S  1  16 2 307 3 244
07/09/84   16:06:44   210 S  1  15 2 308 3 215
07/09/84   16:36:22   210 M  1  15 2 308 3 243
07/09/84   17:06:52   210 S  1  14 2 308 3   .
07/09/84   17:35:30   210 M  1   . 2 309 3 243
07/09/84   18:04:29   210 M  1  32 2 308 3 235
07/09/84   18:52:53   210 M  1   . 2 299 3 222
07/09/84   19:13:19   210 M  1  95 2 298 3 221
07/09/84   19:38:35   210 M  1  92 2 299 3 220
07/09/84   20:05:18   210 M  1 105 2 299 3 220
07/09/84   20:34:08   210 M  1  95 2 294 3 217
07/12/84   00:47:13   210 S  1 165 2 290 3 218
07/12/84   01:09:18   210 M  1   . 2 291 3 219
07/12/84   01:41:44   210 M  1   . 2 290 3 219
;
proc print;
run;
