*-----------------------------------------------------------------*
|   SAS program to plot the von Mises distribution in x,y         |
|      coordinates, and also on a circle.  Note that values of    |
|      s (Standard Deviation) of the distribution should not be   |
|      much smaller than about 10, or numerical overflows will    |
|      result.                                                    |
|                                                                 |
|   Developed by: Gary C. White                                   |
|                 Department of Fishery and Wildlife Biology      |
|                 Colorado State University                       |
|                 Fort Collins, CO 80523                          |
|                 (303)491-6678                                   |
*-----------------------------------------------------------------*;
data vonmises;
   drop pi mur A r p term i0;
   pi=4*atan(1);
   mu=90;   /* Set mean of distribution in degrees */
   mur=mu*pi/180;  /* Set mean in radios also */
   s=10;    /* Set standard deviation of distribution in degrees */
   srad=s*pi/180 /* Set standard deviation in radios also */
   A=exp(-1*(s*pi/180)**2/2);
   kappa=1/(2*(1-A)+((1-A)**2*(0.48794-0.82905*A-1.3915*A*A))/A);
   /* Calculate Bessel function with infinite series summation */
   i0=0;
   p=0;
   do r=0 to 500 by 1;
      term=((kappa/2)**(2*r+p))/(gamma(p+r+1)*gamma(r+1));
      i0=i0+term;
      if (term+i0)/i0 < 1.000001 then goto endloop;
      end;
      put 'ERROR -- Bessel function exceeds 500 terms.';
endloop:
   do theta=-pi to pi by pi/100;
      normpdf=(1./(sqrt(2*pi)*srad))*exp(-1*((theta-mur)**2/(2*srad**2)));
      pdf=exp(kappa*cos(theta-mur))/(2*pi*i0);
      output;
      end;
proc print;
                           /* Print x,y plot of distribution */
proc plot; plot pdf*theta='*' normpdf*theta='n' / overlay;
          /* Develop and print circular plot of distribution */
data polar;
   set vonmises;
      plotsym='+';
      x=cos(theta); y=sin(theta); output;
      plotsym='*';
      x=x*(pdf+1); y=y*(pdf+1); output;
proc plot; plot y*x=plotsym;
run;
