data bear;
   input x y;
cards;
279377 4428990  468.48 -83.012 173.012
279434 4428525 1012.83 -51.897 141.897
280059 4427728  550.01 -74.498 164.498
280206 4427198  149.64 -79.216 169.216
280234 4427051  845.15 -31.057 121.057
280958 4426615  563.39 -87.864 177.864
280979 4426052  765.51 -83.776 173.776
281062 4425291  797.86 -42.206 132.206
281653 4424755  324.85 -30.117 120.117
281934 4424592  754.91   2.809  87.191
282688 4424629  753.35 -84.592 174.592
282759 4423879  819.05 -55.747 145.747
283220 4423202  201.38 -34.479 124.479
283386 4423088  283.95 -111.052 201.052
283284 4422823  783.67 -13.504 103.504
284046 4422640  583.46   9.769  80.231
284621 4422739  228.11 -210.275 300.275
284424 4422854  683.67 -78.953 168.953
284555 4422183  721.01  -0.238  90.238
285276 4422180   30.89  60.945  29.055
285291 4422207  165.70 -51.124 141.124
285395 4422078  909.03 -61.123 151.123
285834 4421282  758.28 -86.673 176.673
285878 4420525  844.79 -26.353 116.353
286635 4420150  .         .      .
;
*------------------------------------------------------------------*
|   SAS code to compute the mean distance between consecutive      |
|   locations, and the mean angle and mean bearing for direction   |
|   between consecutive locations.  The input data file should     |
|   have the variables x and y sorted in chronological order.      |
|   PROC CHART is used to generate histograms of the mean          |
|   direction in 2 different formats for both angles and bearings. |
*----------------------- -------------------------------------------*;
data direct;
   keep x y d a b;
   retain x1 y1 bigxsum bigysum dsum 0;
   set bear end=lastobs;
   x2=x; y2=y;
   if _n_ > 1 then do;
      d=sqrt((x2-x1)**2+(y2-y1)**2);
      link atan2;
      x=x1; y=y1; i=i-1;
      a=alpha; if a<0 then a=a+360; /* Direction of travel as angle */    
      b=90-a; if b<0 then b=b+360;  /* Direction of travel as bearing */      
      output;
      bigxsum=bigxsum+cos(theta);   /* Cumulate sums for means */
      bigysum=bigysum+sin(theta);
      dsum=dsum+d;
   end;
   /* Save current location for next observation */
   x1=x2; y1=y2;
   if lastobs then do;
      x=x2; y=y2; d=.; a=.; b=.;
      output;
      dbar=dsum/(_n_-1);                   /* Average distance */
      bigxbar=bigxsum/(_n_-1);
      bigybar=bigysum/(_n_-1);
      x2=bigxbar; y2=bigybar;
      x1=0; y1=0; link atan2;
      abar=alpha;                      /* Average direction as angle */   
      bbar=90-abar; if bbar<0 then bbar=bbar+360;      /* as bearing */      
      r=sqrt(bigxbar**2+bigybar**2);
      s=sqrt(-2*log(r))*180/pi;
      z=(_n_-1)*r**2;
      mu=-pi/4; /* Set mu to bearing of 135 degrees */
      u=sqrt(2*(_n_-1))*r*cos(theta-mu);
      put 'Mean distance between locations: ' dbar=;
      put '                         Mean X: ' bigxbar=;
      put '                         Mean Y: ' bigybar=;
      put '         Mean direction (angle): ' abar=;
      put '       Mean direction (bearing): ' bbar=;
      put '            Concentration (0-1): ' r=;
      put '             Standard Deviation: ' s=;
      put '          Raleigh''s z statistic: ' z=;
      put '   Expected direction of travel: ' mu=;
      put '      Test statistic for V test: ' u=;
   end;
return;
atan2:  /* SAS code to calculate angle from point (x1, y1)
           to point (x2, y2) */   pi=atan(1.)*4;
        xp=x2-x1; yp=y2-y1;
        if xp > 0 then theta=atan(yp/xp);
        else if xp < 0 then theta=pi+atan(yp/xp);
        else if xp = 0 then do;
           if yp < 0 then theta=pi*3/2;
           else if yp > 0 then theta=pi/2;
           else theta=.; end;
        if theta < 0 then theta=theta+2*pi;
        /* Now convert angle in radians to angle in degrees */
        alpha=theta*(180/pi);
   return;
proc print;
proc gchart;
   vbar a / type=freq midpoints=5 to 355 by 20;
   star a / type=freq midpoints=5 to 355 by 20;
   title 'Angles'; 
proc gchart;
   vbar b / type=freq midpoints=5 to 355 by 20;
   star b / type=freq midpoints=5 to 355 by 20;
   title 'Bearings'; 
run;
